


import Constants,nltk,InvertedIndex,string,DocumentSelection
from sets import Set
import pickle,FSMetric,TextPreprocessingTools
import random


class DocPrepEngine():
    
    def __init__(self):
        self.featureTypes=[Constants.FEATURE_TYPE_BAGS_OF_WORDS,]
        self.featureScoreType=[Constants.TERM_SCORES_TFIDF,]
        self.isStopword=True
        self.isStemming=True
        self.lang="norwegian"
        self.svd_dimension=0
        self.minDocLen=20
        self.nr_bigrams=5000
        self.minTermDocFreq=2
        self.docSelectThresh=5.9
    
    def processDocuments(self,docs,manual=[]): 
        allDocs=docs+manual
        self.tokenize(allDocs)
        print "len docs before min tok len: ",len(docs)
        self.filterOnDocLenght(docs, self.minDocLen)
        print "len docs after min tok len: ",len(docs)
        allDocs=docs+manual
        if self.isStopword: self.stopwordRemoval(allDocs, self.lang)
        if self.isStemming: self.stemming(allDocs, self.lang)
        self.features(allDocs, self.featureTypes)
        
                
        self.filterTermsOnDocFreq(allDocs,self.minTermDocFreq)
        #self.featureExtraction(docs, self.featureTypes)
        #self.featureCount(docs)
        #self.createVocabulary(docs)     
        
        print "docs before doc selection: ",len(allDocs)
        docsel=DocumentSelection.DocumentSelection()
        docsel.selectChi(docs,self.docSelectThresh)
        allDocs=docs+manual
        print "docs after doc selection: ",len(allDocs)
        
        
        
        if Constants.TERM_SCORES_TFIDF in self.featureScoreType: self.tf_idf(allDocs)
        else: self.tf_idf(allDocs)
        if Constants.TERM_SCORES_SVD in self.featureScoreType: self.lsi_svd(allDocs, self.svd_dimension)
        
    def filterTermsOnDocFreq(self,docs,min):
        if len(docs)>0:
            for term in docs[0].invertedIndex.getTerms():
                if docs[0].invertedIndex.getNumberOfDocumentsWithTerm(term)<min: 
                    docs[0].invertedIndex.removeTerm(term)
            print "features after filtering min freq: ", len(docs[0].invertedIndex.getTerms())
                    
    
    def filterOnDocLenght(self,docs,minLen):
        for doc in docs:
            if len(doc.tokens)<minLen:
                docs.remove(doc)
        
    
    def docsToDocTermMatrix(self,docs):
        #print "start doc term matrix"
        #if len(docs)>0: print "Vocab size: ",len(docs[0].vocabulary)
        matrix=[]
        for doc in docs:
            docTermVec=[]
            if doc.term_counts==[]: self.featureCount(docs)
            for term in doc.vocabulary:
                if term in doc.term_counts: docTermVec.append(doc.term_counts[term])
                else: docTermVec.append(0)
            matrix.append(docTermVec)
        #print "end doc term matrix"
        return matrix
                
    def docTermMatrixToDocs(self,docs,matrix):
        vocabulary=[]
        if len(docs)==len(matrix):            
            for i in xrange(len(docs)):
                if vocabulary==[]: vocabulary=list(docs[i].vocabulary)
                docs[i].term_scores={}
                if len(matrix[i])==len(vocabulary):
                    for j in xrange(len(vocabulary)):
                        #print "j = ",j
                        if matrix[i][j] > 0.00000001:
                            docs[i].term_scores[vocabulary[j]]=matrix[i][j]
                        matrix[i][j]=0
                else: print "Docs and doc term matrix not smae number of terms"
        else: print "Docs and doc term matrix not smae number of docs"
            
    
    def tokenize(self,docs):
        s=".:,;*'^<>!\"#%&/()=?}][{@\\"
        for doc in docs:
            doc.tokens = nltk.wordpunct_tokenize(doc.text.lower())  
            doc.tokens = [t for t in doc.tokens if all(c not in t for c in s)]  
            #doc.text=""
            """
            holder=[]
            mem=""
            for token in doc.tokens:                
                if all(c in string.letters for c in token): 
                    holder.append(token)
                    mem=""
                elif all(c in string.digits for c in token):
                    if mem=="-": 
                        holder.append("negativt")
                        holder.append("nummer")
                    else: holder.append("nummer")
                    mem=""
                elif len(token)==1: mem=token
            doc.tokens=holder
            """
            
    
    #TODO WRONG, lengt not same
    def lsi_svd(self,docs,dimensions=0):
        print "SVD"
        matrix=[]
        for doc in docs:
            h=[]
            for term in doc.invertedIndex.getTerms():
                h.append(doc.invertedIndex.getDocTermScore(term,doc.id))
            matrix.append(h)
        matrix=FSMetric.lsi(matrix, dimensions)
        docs[0].invertedIndex.index={}
        for i in xrange(len(matrix)):
            for j in xrange(len(matrix[i])):
                if matrix[i][j]>0.00001:
                    docs[0].invertedIndex.addTerm(docs[i].id,str(j))
                    docs[0].invertedIndex.addScore(docs[i].id,str(j),matrix[i][j])
        print "SVD FINISHED!!!!!!!!!!!"
        #for i in xrange(len())
        
    def stopwordRemoval(self,docs,lang="norwegian"):
        for doc in docs:
            #doc.tokens=TextPreprocessingTools.stopwordremoval(doc.tokens,lang)
            doc.tokens=[token for token in doc.tokens if len(token)>2]
          
                    
        
    def stemming(self,docs,lang="norwegian"):
        for doc in docs:
            doc.tokens=TextPreprocessingTools.stemmer(doc.tokens,lang)
        
    def featureExtraction(self,docs,featureTypes=[Constants.FEATURE_TYPE_BAGS_OF_WORDS,]):
        for doc in docs:
            if Constants.FEATURE_TYPE_BAGS_OF_WORDS in featureTypes: doc.features.extend(doc.tokenized)
            if Constants.FEATURE_TYPE_BIGRAMS in featureTypes:
                #doc.features.extend(nltk.bigrams(doc.tokenized))
                bigram_finder = nltk.BigramCollocationFinder.from_words(doc.tokenized)
                doc.features.extend(bigram_finder.nbest(nltk.BigramAssocMeasures.chi_sq, self.nr_bigrams))
            if doc.features==[]: doc.features.extend(doc.tokenized)
            #doc.tokenized=[]
    
    def features(self,docs,featureTypes=[Constants.FEATURE_TYPE_BAGS_OF_WORDS,]):
        ii=InvertedIndex.InvertedIndex()
        #count=0
        for doc in docs:            
            tokens=doc.tokens
            if Constants.FEATURE_TYPE_BAGS_OF_WORDS in featureTypes:
                for token in tokens:
                    #if token in ft: ft[token]+=1
                    #else: ft[token]=1
                    ii.addTerm(doc.id, token, 1)
            if Constants.FEATURE_TYPE_BIGRAMS in featureTypes:
                for i in xrange(len(tokens)):
                    if i+1<len(tokens)-1:
                        bi=(tokens[i],tokens[i+1])
                        #if bi in ft: ft[bi]+=1
                        #else: ft[bi]=1
                        ii.addTerm(doc.id, bi, 1)
            doc.invertedIndex=ii
            #doc.term_counts=ft
            #count+=1
        #print "FINISH fetures !!!!!!!!"
        print "nr features: ",len(ii.getTerms())
        #print ii.index.keys()
            
    def featureCount(self,docs):
        for doc in docs:
            if doc.features == []: self.featureExtraction(docs)
            doc.term_counts=nltk.FreqDist(doc.features)
            doc.features=[]
            

    def tf_idf(self,docs):
        #matrix=[]
        #for doc in docs:
        #    docTermVec=[]
        #    for term in doc.invertedIndex.index:
        #        docTermVec.append(doc.invertedIndex.getDocTermCount(term,doc.id))
        #    matrix.append(docTermVec)
        #print "matrix before 0: ",matrix[0][:100]
        #print "MATRIX TFIDF FINISH!!!!!!!!"
        FSMetric.tfidf2(docs)  
        #print "matrix after 0: ",matrix[0][:100]     
        print "TFIDF FINISH!!!!!!!!!!!!!!"
        #self.docTermMatrixToDocs(docs, matrix)
        #print "end tfidf"
        #print "term scores in tfidf for doc 0: ",docs[0].term_scores
        #print "term scores in tfidf for doc 0 len: ",len(docs[0].term_scores)
        
        
                
    def createVocabulary(self,docs):
        vocabulary=Set([])
        for doc in docs:
            vocabulary = vocabulary | Set(doc.term_counts.keys())
        vocabulary=list(vocabulary)
        for doc in docs:
            doc.vocabulary=vocabulary
    
    def documentRepresentationsAll(self,docs,vocabulary=[]):
        for doc in docs:
            self.documentRepresentation(doc,vocabulary)
            
            
    def documentRepresentation(self,doc,vocabulary=[]):
        #print doc.term_scores
        doc.doc_rep=[]
        if vocabulary == []: vocabulary=doc.invertedIndex.index.keys()  
        for term in vocabulary:
            doc.doc_rep.append(doc.invertedIndex.getDocTermScore(term,doc.id))  #<---
            
            
        
            
            
    def storePrepDocs(self,docs,tsDocsName):
        file_docs = open(Constants.PATH_DATA+tsDocsName+".obj", 'w') 
        pickle.dump(docs, file_docs)  
    def loadPrepDocs(self,tsDocsName):
        filehandler = open(Constants.PATH_DATA+tsDocsName+".obj", 'r') 
        return pickle.load(filehandler)

    
    def featureReductionChi(self,catDocs,nr):    
        
        return FSMetric.chiSquareTopFeatures(self.divideDocsInPosNeutNeg(catDocs), nr)
        
    def featureReductionChiVocab(self,catDocs,restDocs,nr):
        vocab1=self.featureReductionChi(catDocs,restDocs, nr)
        vocab=[]
        for w in vocab1:
            vocab.append(w[0])
        return vocab
        
    def divideDocsInPosNeutNeg(self,docs):
        pos,nut,neg=[],[],[]
        for doc in docs:
            if doc.category==Constants.SENTIMENT_NEG: neg.append(doc)
            elif doc.category==Constants.SENTIMENT_NUT: nut.append(doc)
            elif doc.category==Constants.SENTIMENT_POS: pos.append(doc)
        return (pos,nut,neg)
    
    
    
    def featureReductionOnTermScores(self,docs,nr):
        termv={}
        if len(docs)>0:
            for term in docs[0].invertedIndex.getTerms():
                termv[term]=docs[0].invertedIndex.getTotalTermScore(term)
            
            holder = []
            for w in sorted(termv, key=termv.get, reverse=True):
                holder.append(w)
            if len(holder)>nr: return holder[:nr]
            else: return holder
            #if len(holder)>nr: holder= holder[:nr]
            #self.createDocumentRepresentation(docs, holder)
        else: return [] 
    

