

import Constants


def nrCorrect(pred,real,label=None):
    if len(pred)==len(real):
        if label==None:
            return sum([1 for i in xrange(len(pred)) if pred[i]==real[i]])
        else:
            return sum([1 for i in xrange(len(pred)) if pred[i]==real[i]==label])
    else:     
        return 0
    

class MultipleClassifierEvaluations():
    def __init__(self,evals):
        self.accuracy=0.0
        self.precision=0.0
        self.recall=0.0
        self.specificity=0.0
        self.f_measure=0.0        
        self.evaluations=evals
        self.crossover={}
        
        self.accuracyAvg()
        self.f_measureAvg()
        self.precisionAvg()
        self.recallAvg()
        self.specificityAvg()
        self.crossoverAvg()
        
    def crossoverAvg(self):
        for e in self.evaluations:
            for label in e.crossover.keys():
                if label not in self.crossover:
                    self.crossover[label]=e.crossover[label]
                else: 
                    self.crossover[label]+=e.crossover[label]
        for label in self.crossover:
            self.crossover[label]=self.crossover[label]/len(self.evaluations)
            
    def precisionAvg(self):
        self.precision= sum([e.precision for e in self.evaluations])/len(self.evaluations)
    def recallAvg(self):
        self.recall= sum([e.recall for e in self.evaluations])/len(self.evaluations)
    def specificityAvg(self):
        self.specificity= sum([e.specificity for e in self.evaluations])/len(self.evaluations)
    def f_measureAvg(self):
        self.f_measure= sum([e.f_measure for e in self.evaluations])/len(self.evaluations)
    def accuracyAvg(self):
        self.accuracy=sum([e.accuracy for e in self.evaluations])/len(self.evaluations)
    
    def toStringEval(self,name=""):
        s="___Evaluation "+str(name)+"____"
        #s+= "\nTrue Positive: "+str(self.true_positive)
        #s+= "\nTrue Negative: "+str(self.true_negative)
        #s+= "\nFalse Positive: "+str(self.false_positive)
        #s+= "\nFalse Negative: "+str(self.false_negative)
        s+= "\nAccuracy: "+str(self.accuracy)
        s+= "\nPrecision: "+str(self.precision)
        s+= "\nRecall: "+str(self.recall)
        s+= "\nF-Measure: "+str(self.f_measure)
        #s+= "\nSpecificity: "+str(self.specificity)
        s+= "\nCrossover: "
        for label in self.crossover:
            s+= str(label)+": "+str(self.crossover[label])+" "
        s+= "\n___END OF Evaluation "+str(name)+"____"
        return s
    
class ClassifierEvaluation():
    def __init__(self,pred,real,beta=1.0):
        #self.accuracyCalc=self.accuracyCalc(pred, real)
        #self.precision=self.precisionCalc(pred, real)
        self.pos_eval=BinaryClassifierEvaluation(pred,real,Constants.SENTIMENT_POS,beta)
        self.nut_eval=BinaryClassifierEvaluation(pred,real,Constants.SENTIMENT_NUT,beta)
        self.neg_eval=BinaryClassifierEvaluation(pred,real,Constants.SENTIMENT_NEG,beta)
        
    def toStringEval(self,name=""):
        pass
    """
    #TODO: is this correct? Remove? Not of interest?
    def accuracyCalc(self,pred,real):
        return (self.pos_eval.accuracy+self.nut_eval.accuracy+self.neg_eval.accuracy)/3
    def precisionCalc(self,pred,real):
        return (self.pos_eval.precision+self.nut_eval.precision+self.neg_eval.precision)/3
    def recallCalc(self,pred,real):
        return (self.pos_eval.recall+self.nut_eval.recall+self.neg_eval.recall)/3
    def specificityCalc(self,pred,real):
        return (self.pos_eval.specificity+self.nut_eval.specificity+self.neg_eval.specificity)/3
    def f_measureCalc(self,pred,real):
        return (self.pos_eval.f_measure+self.nut_eval.f_measure+self.neg_eval.f_measure)/3
    
    """
    
    
###########
#Binary stuff
class BinaryClassifierEvaluation():
    
    def __init__(self,pred,real,label,beta=1.0):
        self.true_positive=self.truePositive(pred, real, label)
        self.false_positive=self.falsePositive(pred, real, label)
        self.true_negative=self.trueNegative(pred, real, label)
        self.false_negative=self.falseNegative(pred, real, label)
        self.precision = self.precisionCalc(pred, real, label)
        self.recall = self.recallCalc(pred, real, label)
        self.specificity = self.specificityCalc(pred, real, label)
        self.accuracy = self.accuracyCalc(pred, real, label)
        #self.beta=beta
        self.f_measure = self.f_measureCalc(pred, real, label,beta)
        
        
        self.crossover=self.crossoverCalc(pred, real, label)
    
    def crossoverCalc(self,pred,real,label):  
        if len(pred)==len(real):
            if self.true_positive>0:
                labels=[]
                for l in real:
                    if l not in labels: labels.append(l)
                labelsCount={}
                for i in xrange(len(pred)):
                    if real[i]!=label==pred[i]: #if doc is label and pred is not 
                        if real[i] not in labelsCount: labelsCount[real[i]]=1
                        else: labelsCount[real[i]]+=1
                    #TODO: add tp and FP count
                for l in labelsCount.keys():                
                    labelsCount[l]=float(labelsCount[l])/(self.true_positive+self.false_positive)  
                return labelsCount        
        return {}
    
    def truePositive(self,pred,real,label):
        if len(pred)==len(real):
            return sum([1 for i in xrange(len(pred)) if pred[i]==real[i]==label])
        else:     
            return 0
    
    def falsePositive(self,pred,real,label):
        if len(pred)==len(real):
            return sum([1 for i in xrange(len(pred)) if pred[i]==label!=real[i]])
        else:     
            return 0
    
    def trueNegative(self,pred,real,label):
        if len(pred)==len(real):
            return sum([1 for i in xrange(len(pred)) if pred[i]!=label!=real[i]])
        else:     
            return 0
    
    def falseNegative(self,pred,real,label):
        if len(pred)==len(real):
            return sum([1 for i in xrange(len(pred)) if real[i]==label!=pred[i]])
        else:     
            return 0
        
    def precisionCalc(self,pred,real,label):
        try:
            tp=1.0*self.truePositive(pred,real,label)
            return tp / (tp+self.falsePositive(pred,real,label))
        except:
            return 0.0
        
    def recallCalc(self,pred,real,label):
        try:
            tp=1.0*self.truePositive(pred,real,label)
            return tp / (tp+self.falseNegative(pred,real,label))
        except:
            return 0.0
    
    def specificityCalc(self,pred,real,label):
        try:
            tn = 1.0*self.trueNegative(pred,real,label)
            return tn / (tn+self.falsePositive(pred,real,label))
        except:
            return 0.0
    
    def accuracyCalc(self,pred,real,label):
        try:
            tn = 1.0*self.trueNegative(pred,real,label)
            tp=1.0*self.truePositive(pred,real,label)
            return (tp+tn)/(len(pred))
        except:
            return 0.0
        
    def f_measureCalc(self,pred,real,label,beta=1.0):
        try:
            p = 1.0*self.precisionCalc(pred,real,label)
            r = 1.0*self.recallCalc(pred,real,label)
            return (1+float(beta)**2) * (1.0*(p*r) / ((float(beta)**2)*p + r))
        except:
            return 0.0
        
    def toStringEval(self,name=""):
        s="___Evaluation "+str(name)+"____"
        s+= "\nTrue Positive: "+str(self.true_positive)
        s+= "\nTrue Negative: "+str(self.true_negative)
        s+= "\nFalse Positive: "+str(self.false_positive)
        s+= "\nFalse Negative: "+str(self.false_negative)
        s+= "\nAccuracy: "+str(self.accuracy)
        s+= "\nPrecision: "+str(self.precision)
        s+= "\nRecall: "+str(self.recall)
        s+= "\nF-Measure (beta="+str(self.beta)+"): "+str(self.f_measure)
        #s+= "\nSpecificity: "+str(self.specificity)
        s+= "\n___END OF Evaluation "+str(name)+"____"
        return s
#########




    
    
    
    
    
    
    
    
    
    